"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const renderer_1 = require("./renderer");
const Path = require("path");
const deepmerge = require("deepmerge");
const fs = require("fs-extra-plus");
// --------- /Public Types --------- //
// --------- Public Loaders --------- //
async function loadVersionFiles() {
    const rawConfig = await loadVdevConfig();
    return rawConfig.versionFiles || [];
}
exports.loadVersionFiles = loadVersionFiles;
const overwriteMerge = (target, source, options) => source;
/** Parse the ./vdev.yaml and return as is */
async function loadVdevConfig(rootDir) {
    rootDir = rootDir || "./";
    const vdevFile = Path.join(rootDir, 'vdev.yaml');
    // NOTE: Here we use the loadTempletezedYaml, but for now, the vdev files are not templates, just the k8s yamls are.
    let vdevObj = await loadVdevFile(vdevFile, true);
    // TODO: probably need to do some validate here.
    // load the sub vdev files if defined
    if (vdevObj.vdevFiles) {
        for (const subRelVdevFile of vdevObj.vdevFiles) {
            const subVdevFile = Path.join(rootDir, subRelVdevFile);
            const exists = await fs.pathExists(subVdevFile);
            if (exists) {
                console.log(`INFO - Loading extra vdev file: ${subVdevFile}`);
                try {
                    let subVdevObj = await loadVdevFile(subVdevFile);
                    vdevObj = deepmerge(vdevObj, subVdevObj, { arrayMerge: overwriteMerge });
                }
                catch (ex) {
                    console.log(`ERROR - Cannot load vdev files ${subVdevFile} (skipping file)\n\tCause: ${ex}`);
                }
            }
        }
    }
    return vdevObj;
}
exports.loadVdevConfig = loadVdevConfig;
// --------- /Public Loaders --------- //
/**
 * Load a single vdev yaml file. Can be the base or a sub vdev file
 * @param base If the vdev file is the base (should have more validation)
 */
async function loadVdevFile(vdevFile, base = false) {
    const vdevRawObj = await renderer_1.loadTemplatizedYaml(vdevFile);
    // TODO: need to do validation
    return vdevRawObj;
}
